#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

set -eu

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error "Could not determine script path"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/../../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error "Could not determine modules path"
fi

INFRA_DIR="${SCRIPT_DIR}/../"

readonly SCRIPT_DIR
readonly MODULES_DIR
readonly INFRA_DIR

#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/utils.sh
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}"/longhorn_helper.sh
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}"/disaster_recovery_utils.sh
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}"/volume_restore.sh

# kube config file
RESTORE_DIR="/mnt/restore"
export MERGEDINPUTFILE="${SCRIPT_DIR}/merged_infra_input.json"
export NODE_ROLE=${NODE_ROLE:-none}
export RESTORE_VOLUME=${RESTORE_VOLUME:-false}
export RESTORE_CLUSTERCONFIG_INPUTFILE="${SCRIPT_DIR}/restore_cluster_config.json"

########
# Parse input json file to set the config values
# Globals:
#   INFILE
# Outputs:
#   Sets global config variables
########
function parse_input_file() {
  set -e
  ETCD_RESTORE_PATH="$(get_mandatory_json_value ".restore.etcdRestorePath")"
  ARGOCD_VERSION="$(get_mandatory_json_value ".fabric.argocd_version")"
  STORAGE_CLASS="$(get_mandatory_json_value ".storage_class")"
  STORAGE_CLASS_SINGLE_REPLICA="$(get_mandatory_json_value ".storage_class_single_replica")"
  PROXY_ENABLED="$(get_optional_json_value ".proxy.enabled")"
  HTTP_PROXY="$(get_optional_json_value ".proxy.http_proxy")"
  NFS_ENDPOINT="$(get_mandatory_json_value ".restore.nfs.endpoint")"
  NFS_MOUNT_PATH="$(get_mandatory_json_value ".restore.nfs.mountpath")"
  NFS_SERVER_PORT="$(get_mandatory_json_value ".backup.nfs.port")"

  export ARGOCD_VERSION="${ARGOCD_VERSION}"
  export STORAGE_CLASS="${STORAGE_CLASS}"
  export PROXY_ENABLED="${PROXY_ENABLED}"
  export HTTP_PROXY="${HTTP_PROXY}"
  export STORAGE_CLASS="${STORAGE_CLASS}"
  export STORAGE_CLASS_SINGLE_REPLICA="${STORAGE_CLASS_SINGLE_REPLICA}"
  set +e
}

########
# Parse restore json file and expose required param
# Globals:
#   INFILE
# Outputs:
#   Sets global config variables
########
function parse_restored_input_file() {
  set -e
  FQDN="$(get_mandatory_json_value ".fqdn")"
  PROFILE="$(get_mandatory_json_value ".profile")"
  USE_CILIUM="$(get_optional_json_value ".use_cilium")"
  if [[ "${USE_CILIUM}" == "" ]]; then
    USE_CILIUM="0"
  fi

  export FQDN="${FQDN}"
  export PROFILE="${PROFILE}"
  export USE_CILIUM="${USE_CILIUM}"
  set +e
}

function restart_pods_with_servicetoken() {
  local secret_list
  info "Cleaning up pods with service token..."
  secret_list=$(kubectl get secrets -o json -A|jq -c ".items[]|select(.type==\"kubernetes.io/service-account-token\")| {name: .metadata.name, namespace: .metadata.namespace}")
  for secret in ${secret_list}; do
    local secret_name
    local secret_namespace
    local pod_list
    secret_name=$(echo "${secret}"|jq -r ".name")
    secret_namespace=$(echo "${secret}"|jq -r ".namespace")
    pod_list=$(kubectl get pods -n "${secret_namespace}" -o json |jq -c ".items[]| {name: .metadata.name, secretname: .spec | select( has (\"volumes\") ).volumes[]| select( has (\"secret\") ).secret| select( .secretName == \"${secret_name}\" )|.secretName}")

    kubectl delete secret "${secret_name}" -n "${secret_namespace}"

    echo "Pod List attached to secret: ${secret_name}: ${pod_list}"
    echo "kubectl get pods -n \"${secret_namespace}\" -o json |jq -c \".items[]| {name: .metadata.name, secretname: .spec | select( has (\"volumes\") ).volumes[]| select( has (\"secret\") ).secret| select( .secretName == \"${secret_name}\" )|.secretName}\""
    for pod in ${pod_list}; do
      local pod_name
      pod_name=$(echo "${pod}"|jq -r ".name")
      kubectl delete pod "${pod_name}" -n "${secret_namespace}" --force
    done
  done
  info "Successfully cleanup for pods."
}

function remove_inactive_nodes() {
  local inactive_node_names current_node_name
  current_node_name=$(hostname)
  sleep 120
  inactive_node_names=$(kubectl get nodes|grep "NotReady"|awk '{print $1}')
  for inactive_node in ${inactive_node_names}; do
    if [[ "${current_node_name,,}" != "${inactive_node,,}" ]] ; then
      kubectl cordon "${inactive_node}"
      kubectl delete node "${inactive_node}"
    fi
  done
}

########
# Restore ETCD Snapshots
########
function restore_etcd_snapshot(){
  info "Restoring ETCD from ${ETCD_RESTORE_PATH} started..."

  # install rke2
  "${INFRA_DIR}"/infra-installer.sh -i "${INFILE}" -o "${INFRA_DIR}/restore_infra_output.json"

  # stop rke2-server before etcd restore
  stop_rke2_server

  # create dir if not exist
  mkdir -p /var/lib/rancher/rke2/server/db || true
  mkdir -p /var/lib/kubelet || true
  # restore mount path
  mount -a || true

  local local_etcd_restorepath
  local_etcd_restorepath=${RESTORE_DIR}/${ETCD_RESTORE_PATH}/

  local latest_snapshot
  local restore_file
  # shellcheck disable=SC2012
  latest_snapshot=$(ls -t1 "${local_etcd_restorepath}/snapshots"|  head -n 1)

  restore_file=${local_etcd_restorepath}/snapshots/${latest_snapshot}
  info "Restore ETCD with snapshot ${latest_snapshot} from ${restore_file} started..."

  if [ ! -f "${restore_file}" ]; then
    error "Restore file not found at ${restore_file}"
  fi

  rke2 server --cluster-reset --cluster-reset-restore-path="${restore_file}" &> rke2-cluster-reset.logs
  rke2-killall.sh
  # verify kubelet killed
  kill_kubelet_containerd
  # restore mount path
  mount -a || true

  start_rke2_server

  sleep 60

  validate_cluster_ready 50 "${NODE_ROLE}"

  disable_pvc_backup

  remove_inactive_nodes

  # scale down rook-ceph
  kubectl -n rook-ceph scale deployment rook-ceph-operator --replicas=0 || true

  containercreatingcount=-1
  try=0
  maxtry=60
  while(( try < maxtry )); do
    # kubectl get pods -A | grep -v "Running"| grep -v "Complete"| grep -v "Terminating"; sleep 10
    try=$(( try + 1 ))
    containercreatingcount=$(kubectl get pods -A | grep -c "ContainerCreating")
    info "Total pods in ContainerCreating ${containercreatingcount}, Waiting for all pods to come up... ${try}/${maxtry}"; sleep 10
    if [[ "${containercreatingcount}" -le 2 ]]; then
      break
    fi
  done

  set_longhorn_url "${NODE_ROLE}"
  validate_longhorn_reachable ; sleep 120
  # update longhorn config
  update_longhorn_config

  # reset settings for backup
  update_default_settings "backup-target" "" "ignore_failure"

  info "Restoring ETCD successfull."
}

function validate_and_mount_nfs_points() {
  local nfs_end_point=$1
  local mount_path=$2

  validate_nfs_server_connection "${nfs_end_point}" "${mount_path}" "${NFS_SERVER_PORT}"

  # mount nfs server to client
  mkdir -p ${RESTORE_DIR}

  # backup location is <mount_path>/backup for etcd and data backups
  mount_result=$(mount -t nfs "${nfs_end_point}:${mount_path}/backup" ${RESTORE_DIR} 2>&1)
  if { [ -n "${mount_result}" ] || [ "${mount_result}" == " " ] ; }; then
    error "Mount Path not exposed for nfs server ${nfs_end_point}"
  fi
}

########
# Validate NFS Server Connection and Mount Path
########
function validate_nfs_server_connection_and_mountpath() {
  local local_etcd_restorepath
  local_etcd_restorepath=${RESTORE_DIR}/${ETCD_RESTORE_PATH}/

  # validate if etcd backup exists
  if ! { [ -d "${local_etcd_restorepath}" ] ; }; then
    error "Backup not available in backup store."
  fi

  if [[ "${NODE_ROLE}" != "agent" && "${RESTORE_VOLUME}" != "true" ]]; then
    # validate all backup folders exits
    if { ! [ -d "${local_etcd_restorepath}/snapshots" ] || [ -z "$(ls -A "${local_etcd_restorepath}"/snapshots/*etcd-snapshot*)" ] ; }; then
      error "ETCD Snapshot Backup not available in backup store ${local_etcd_restorepath}/snapshots."
    fi
  fi

  info "NFS Server connection & mountpath successfully validated."
}

########
# Restore Cluster with NFS Server
########
function restore_cluster_with_nfs_server(){
  info "Start Restoring Cluster..."
  # validate connection with nfs server and mount path
  validate_nfs_server_connection_and_mountpath

  if [[ "${NODE_ROLE}" == "none" ]]; then
    if [[ "${RESTORE_VOLUME}" == "true"  ]]; then
      # validate and restore PVC's and Deployments
      restore_pvcs_and_dependent_deployments
    else
      # restore etcd snapshot
      restore_etcd_snapshot
    fi
  else
    # install rke2
    "${INFRA_DIR}"/infra-installer.sh -i "${INFILE}" -o "${INFRA_DIR}/restore_infra_output.json"

  fi
}

########
# Copy backup files
########
function copy_backup_cluster_config(){
  local tmp
  local local_etcd_restorepath
  local_etcd_restorepath=${RESTORE_DIR}/${ETCD_RESTORE_PATH}/

  if [[ -f "${local_etcd_restorepath}/cluster_config.json" ]]; then
    cp -r "${local_etcd_restorepath}"/cluster_config.json "${RESTORE_CLUSTERCONFIG_INPUTFILE}"
    tmp=$(mktemp)
    jq --arg nfs_end_point "${NFS_ENDPOINT}" '.restore.nfs.endpoint = $nfs_end_point' "${RESTORE_CLUSTERCONFIG_INPUTFILE}" > "${tmp}" && mv "${tmp}" "${RESTORE_CLUSTERCONFIG_INPUTFILE}"
    jq --arg mount_path "${NFS_MOUNT_PATH}" '.restore.nfs.mount_path = $mount_path' "${RESTORE_CLUSTERCONFIG_INPUTFILE}" > "${tmp}" && mv "${tmp}" "${RESTORE_CLUSTERCONFIG_INPUTFILE}"
    jq --arg etc_restore_path "${ETCD_RESTORE_PATH}" '.restore.etcdRestorePath = $etc_restore_path' "${RESTORE_CLUSTERCONFIG_INPUTFILE}" > "${tmp}" && mv "${tmp}" "${RESTORE_CLUSTERCONFIG_INPUTFILE}"
    INFILE="${RESTORE_CLUSTERCONFIG_INPUTFILE}"
  else
    if [[ "${RESTORE_VOLUME}" != "true"  ]]; then
      error "Restore Cluster config file not present on nfs server..."
    fi
  fi
}

#MAIN
function main() {
  parse_args "$@"

  merge_default_values
  parse_input_file
  info "Start validating nfs server and mounting nfs server path..."
  validate_and_mount_nfs_points "${NFS_ENDPOINT}" "${NFS_MOUNT_PATH}"
  info "Preparing input file for restore..."

  copy_backup_cluster_config
  merge_default_values
  if [[ "${RESTORE_VOLUME}" != "true"  ]]; then
    parse_restored_input_file
  fi
  
  parse_fqdn_values

  export ETCD_RESTORE="true"
  # restore cluster
  restore_cluster_with_nfs_server
  # unmount restore path
  umount "${RESTORE_DIR}" || true
  info "Cluster node successfully restored..."
}

main "$@"