#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

function generate_v3_file() {
    local v3_file=$1
    local fqdn=$2
    cat <<EOF > "${v3_file}"
authorityKeyIdentifier=keyid,issuer
basicConstraints=CA:FALSE
keyUsage = digitalSignature, nonRepudiation, keyEncipherment, dataEncipherment
extendedKeyUsage = serverAuth
subjectAltName = @alt_names
[alt_names]
DNS.1 = ${fqdn}
DNS.2 = *.${fqdn}
EOF
}

function generate_openssl_cnf_file() {
  folder_path="$1"
  #shellcheck disable=SC2016
  echo '
[ ca ]
default_ca = CA_default
[ CA_default ]
dir            = '"${folder_path}"'           # Where everything is kept
certs          = $dir/certs               # Where the issued certs are kept
crl_dir        = $dir/crl                 # Where the issued crl are kept
database       = $dir/index.txt           # database index file.
new_certs_dir  = $dir/certs               # default place for new certs.
certificate    = $dir/certs/ca.crt        # The CA certificate
serial         = $dir/serial              # The current serial number
crl            = $dir/crl.pem             # The current CRL
private_key    = $dir/private/ca.key      # The private key
RANDFILE       = $dir/.rnd                # private random number file
nameopt        = default_ca
certopt        = default_ca
policy         = policy_match
default_days   = 365
default_md     = sha256

[ policy_match ]
countryName            = optional
stateOrProvinceName    = optional
organizationName       = optional
organizationalUnitName = optional
commonName             = supplied
emailAddress           = optional

[req]
req_extensions = v3_req
distinguished_name = req_distinguished_name

[req_distinguished_name]

[v3_req]
basicConstraints = CA:TRUE
' > "${folder_path}/openssl.cnf"
}

function create_certs_related_folder_and_files() {
  local folder="$1"
  mkdir -p "${folder}"
  mkdir "${folder}/certs" "${folder}/csr" "${folder}/private"
  echo 1000 > "${folder}/serial"
  touch "${folder}/index.txt"
  generate_openssl_cnf_file "${folder}"
}

function create_self_issued_certificate(){
    local fqdn=$1
    local root_ca_cert_file="$2"
    local tls_cert_file="$3"
    local tls_key_file="$4"
    local root_ca_subj="/C=US/ST=NY/O=UiPath, Inc./CN=UiPath AS Root CA"
    local intermedite_ca_subj="/C=US/ST=NY/O=UiPath, Inc./CN=UiPath AS Intermediate CA"
    local tls_cert_subj="/C=US/ST=NY/O=UiPath, Inc./CN=UiPath Automation Suite"
    local key_length=2048
    local ca_expiry=365
    local intermediate_expiry=300 # Generally intermediate cert expiry should be less than root
    local tls_expiry=90
    local root_ca_folder_path
    local intermediate_ca_folder_path
    local server_ca_folder_path
    local tmp_dir
    local pass
    pass=$(tr </dev/urandom -dc _A-Z-a-z-0-9 | head -c 16)

    CERT_DIR="$(dirname "${root_ca_cert_file}")"
    [[ -d ${CERT_DIR} ]] || mkdir -p "${CERT_DIR}"

    local v3_file="${CERT_DIR}/v3.txt"
    tmp_dir=$(mktemp -d certificates.XXXXXXX)

    # Generate root CA
    root_ca_folder_path="${tmp_dir}/root"
    create_certs_related_folder_and_files "${root_ca_folder_path}"
    openssl genrsa -des3 -passout pass:"$pass" -out "${root_ca_folder_path}/private/root.ca.key" "${key_length}"
    openssl req -new -x509 -days "${ca_expiry}" -passin pass:"$pass" -config "${root_ca_folder_path}/openssl.cnf" -sha256 -extensions v3_req -key "${root_ca_folder_path}/private/root.ca.key" -out "${root_ca_folder_path}/certs/root.ca.crt" -subj "${root_ca_subj}"

    # Generate intermediate CA
    intermediate_ca_folder_path="${tmp_dir}/intermediate"
    create_certs_related_folder_and_files "${intermediate_ca_folder_path}"
    openssl genrsa -des3 -passout pass:"$pass" -out "${intermediate_ca_folder_path}/private/intermediate.ca.key" "${key_length}"
    openssl req -new -sha256 -config "${intermediate_ca_folder_path}/openssl.cnf" -passin pass:"$pass" -key "${intermediate_ca_folder_path}/private/intermediate.ca.key" -out "${intermediate_ca_folder_path}/csr/intermediate.ca.csr" -subj "${intermedite_ca_subj}"
    openssl ca -batch -config "${root_ca_folder_path}/openssl.cnf" -extensions v3_req -keyfile "${root_ca_folder_path}/private/root.ca.key" -cert "${root_ca_folder_path}/certs/root.ca.crt" -md sha256 -days "${intermediate_expiry}" -notext  -passin pass:"$pass" -in "${intermediate_ca_folder_path}/csr/intermediate.ca.csr" -out "${intermediate_ca_folder_path}/certs/intermediate.ca.crt"
    cat "${intermediate_ca_folder_path}/certs/intermediate.ca.crt" "${root_ca_folder_path}/certs/root.ca.crt" > "${intermediate_ca_folder_path}/certs/ca_bundle.crt"
    #shellcheck disable=SC2015
    openssl verify -CAfile "${root_ca_folder_path}/certs/root.ca.crt" "${intermediate_ca_folder_path}/certs/ca_bundle.crt" 2>/dev/null >/dev/null && info "openssl verification passed for intermediate cert" || error "openssl verification failed because generated intemediate cert is not valid for generated rootCA"

    # Generate server cert
    server_ca_folder_path="${tmp_dir}/server"
    generate_v3_file "$v3_file" "$fqdn" 
    create_certs_related_folder_and_files "${server_ca_folder_path}"
    openssl genrsa -out "${server_ca_folder_path}/private/server.key" "${key_length}"
    openssl req -new -key "${server_ca_folder_path}/private/server.key" -out "${server_ca_folder_path}/csr/server.csr" -subj "${tls_cert_subj}"
    openssl x509 -req -in "${server_ca_folder_path}/csr/server.csr" -passin pass:"$pass" -CA "${intermediate_ca_folder_path}/certs/ca_bundle.crt" -CAkey "${intermediate_ca_folder_path}/private/intermediate.ca.key" -out "${server_ca_folder_path}/certs/server.crt" -CAcreateserial -days ${tls_expiry} -sha256 -extfile "${v3_file}"
    #shellcheck disable=SC2015
    openssl verify -CAfile "${intermediate_ca_folder_path}/certs/ca_bundle.crt" "${server_ca_folder_path}/certs/server.crt"  2>/dev/null >/dev/null && info "openssl verification passed for server cert" || error "openssl verification failed because generated server cert is not valid for generated ca bundle"

    cp "${intermediate_ca_folder_path}/certs/ca_bundle.crt" "${root_ca_cert_file}"
    cp "${server_ca_folder_path}/certs/server.crt" "${tls_cert_file}"
    cp "${server_ca_folder_path}/private/server.key" "${tls_key_file}"

    rm -rf "${tmp_dir}"
    info "TLS certificate generated..."
}

function create_token_signing_certificate(){
    local cert_file="$1"
    local key_file="$2"
    local out_path="$3"
    local pass="$4"

    if ! openssl pkcs12 -export -out "$out_path" -inkey "$key_file" -in "$cert_file" --passout pass:"$pass" 2> /dev/null; then
        error "Unable to create token signing certificate"
    fi

    info "Token signing certificate generated..."
}

function validate_certificate_key_match() {
  local cert_file=$1
  local cert_key=$2
  
  cert_checksum=$(openssl x509 -noout -modulus -in "${cert_file}" 2> /dev/null | openssl md5)
  key_checksum=$(openssl rsa -noout -modulus -in "${cert_key}" 2> /dev/null | openssl md5)

  if [[ "${cert_checksum}" == "${key_checksum}" ]]; then 
    echo ""
  else
    echo "Certificate and Key checksum is mismatched."
  fi  
}

function validate_certificate_expiry() {
  local cert_file=$1
  expiry=$(openssl x509 -in "${cert_file}" -noout -checkend 86400 2> /dev/null)

  if [[ "${expiry}" == "Certificate will not expire" ]]; then
    echo ""
  else
   echo "Certificate already expired or will expire within 24hours"
  fi
}

function validate_certificate_key_length() {
  local cert_key=$1
  length=$(openssl rsa -in "${cert_key}" -text -noout 2> /dev/null | grep "Private-Key" | awk '{print $3}' | sed -e 's/(//g')

  if [[ ${length} -ge 2048 ]]; then
    echo ""
  else
    echo "Ceritifcate Key should be minimum of 2048, current length is ${length}"
  fi
}

function validate_certificate_san() {
  local cert_file=$1
  local fqdn=$2
  
  server_dns=$(openssl x509 -in "${cert_file}" -noout -text 2> /dev/null | grep "DNS:" | sed -e "s/DNS://g" | sed -e "s/[[:space:]]//g")
  
  IFS="," read -r -a dns_arr <<< "$server_dns"
  for dns in "${dns_arr[@]}"
  do
    if [[ "$dns" == "$fqdn" ]]; then
      echo ""
      return 0
    fi
  done
  
  echo "Certificate ${cert_file} doesn't have ${fqdn} in SAN"
}

########
# $1 should be crt file path
########
function validate_pem_format() {
  pem_file_path=$1
  openssl x509 -in "${pem_file_path}" -noout -text >/dev/null 2>&1 && echo "" || echo "${pem_file_path} is not in pem format"
}

########
# $1 should be tls crt file path
########
function validate_tls_has_one_certificate() {
  pem_file_path=$1
  local count
  count="$(grep -o "BEGIN CERTIFICATE" "${pem_file_path}" | wc -l)"
  [[ "${count}" == 1 ]] && echo "" || echo "${pem_file_path} should have only one certificate, not chain of certificate"
}

function validate_rootca_match_tls() {
  local ca_file=$1
  local tls_file=$2
  openssl verify -CAfile "${ca_file}" "${tls_file}" 2>/dev/null >/dev/null && echo "" || echo "Either partial ca chain has been provided or leaf server certificate is not signed by provided ca"
}

function validate_server_certificate() {

  local ca_file_path=$1
  local cert_file_path=$2
  local key_file_path=$3
  local fqdn=$4
  local argo_fqdn=$5
  local objectstore_fqdn=$6
  local registry_fqdn=$7
  local monitoring_fqdn=$8
  local insights_fqdn=$9
  local message
  message="success"

  if [[ ! -f "${ca_file_path}" ]]; then
      server_cert_error+=("CA Bundle Certificate file not found: ${ca_file_path}")
  fi

  if [[ ! -f "${cert_file_path}" ]]; then
      server_cert_error+=("Server Certificate file not found: ${cert_file_path}")
  fi

  if [[ ! -f "${key_file_path}" ]]; then
      server_cert_error+=("Server Certificate Private Key file not found: ${key_file_path}")
  fi

  [[ ${#server_cert_error[@]} -ne 0 ]] && failure_output "Validating Certificate..... failed" "${server_cert_error[@]}"
  
  response=$(validate_pem_format "${cert_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")

  response=$(validate_pem_format "${ca_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")

  response=$(validate_certificate_key_match "${cert_file_path}" "${key_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")
  
  response=$(validate_certificate_expiry "${cert_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")
  
  response=$(validate_certificate_key_length "${key_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")
  
  response=$(validate_tls_has_one_certificate "${cert_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")

  response=$(validate_rootca_match_tls "${ca_file_path}" "${cert_file_path}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")
  
  response=$(validate_certificate_san "${cert_file_path}" "${fqdn}")
  [[ -n ${response} ]] && server_cert_error+=("${response}")

  response=$(validate_certificate_san "${cert_file_path}" "*.${fqdn}")
  [[ -n ${response} ]] && wild_card_san_error+=("${response}")
  
  if [[ -n "${response}" ]]; then
     response=$(validate_certificate_san "${cert_file_path}" "${argo_fqdn}")
     [[ -n ${response} ]] && subdomain_san_error+=("${response}")

     response=$(validate_certificate_san "${cert_file_path}" "${objectstore_fqdn}")
     [[ -n ${response} ]] && subdomain_san_error+=("${response}")

    [[ -n "${registry_fqdn}" ]] && response=$(validate_certificate_san "${cert_file_path}" "${registry_fqdn}") || response=""  
    [[ -n ${response} ]] && subdomain_san_error+=("${response}")

     response=$(validate_certificate_san "${cert_file_path}" "${monitoring_fqdn}")
     [[ -n ${response} ]] && subdomain_san_error+=("${response}")

     response=$(validate_certificate_san "${cert_file_path}" "${insights_fqdn}")
     [[ -n ${response} ]] && subdomain_san_error+=("${response}")
  fi

  if [[ ${#subdomain_san_error[@]} -ne 0 && ${#wild_card_san_error[@]} -ne 0 ]]; then
    server_cert_error+=("${subdomain_san_error[@]}")
    server_cert_error+=("${wild_card_san_error[@]}")
  fi

  if [[ -n $UPDATE_FQDN && "$UPDATE_FQDN" == "1" ]];
  then [[
     ${#server_cert_error[@]} -ne 0 ]] && warn_output "Validating Certificate..... failed" "${server_cert_error[@]}" && return
  elif [[ ${#server_cert_error[@]} -ne 0 ]]; then
    failure_output "Validating Certificate..... failed" "${server_cert_error[@]}"
    message="failed"
  fi

  # donot remove this line, as calling function is dependent on this line to perform futher steps
  echo -n "${message}"
}

function validate_signing_certificate(){
    local cert_file=$1
    local pass=$2

    if [[ ! -f "${cert_file}" ]]; then
      token_cert_error+=("token signing certificate file not found: ${cert_file}")
    fi

    [[ ${#token_cert_error[@]} -ne 0 ]] && failure_output "Validating Token Signing Certificate..... failed" "${token_cert_error[@]}"

    if token_cert=$(openssl pkcs12 -in "$cert_file" -nodes --passin pass:"$pass" 2> /dev/null); then
        expiry=$(echo "${token_cert}" | openssl x509 -noout -checkend 86400)
        [[ "${expiry}" != "Certificate will not expire" ]] && token_cert_error+=("Ceritifcate already expired or will expire within 24hours")
    else
        token_cert_error+=("Password is invalid for certificate")
    fi

    [[ ${#token_cert_error[@]} -ne 0 ]] && failure_output "Validating Token Signing Certificate..... failed" "${token_cert_error[@]}"

    echo -n "success"
}
