#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/utils.sh

#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}"/cluster_utils.sh

function configure_cilium() {
  local rke2ConfigurationDir="/var/lib/rancher/rke2/server/manifests"
  local rke2CiliumConfigurationPath="${rke2ConfigurationDir}/rke2-cilium-config.yaml"
  cat > "${rke2CiliumConfigurationPath}" <<EOF
apiVersion: helm.cattle.io/v1
kind: HelmChartConfig
metadata:
  name: rke2-cilium
  namespace: kube-system
spec:
  valuesContent: |-
    cilium:
      operator:
        replicas: 1
      kubeProxyReplacement: "disabled"
EOF
}

function is_cilium_installed() {
  # Check that the node is configured to use Cilium (only on Server nodes)
  local config_path="/etc/rancher/rke2/config.yaml"
  if [[ ("${NODE_ROLE}" == "none" || "${NODE_ROLE}" == "server") && "$(grep cilium -c ${config_path})" -eq 0 ]]; then
    echo "false"
    return
  fi

  # Check that Cilium was installed in the cluster
  if [[ "$(kubectl -n kube-system get ds cilium --ignore-not-found -o name | wc -l)" -eq 0 ]]; then
    echo "false"
    return
  fi

  # Check that this node was properly rebooted after Cilium was installed
  if [[ $(ip link list | grep -c cali) -gt 0 ]]; then
    echo "false"
    return
  fi

  echo "true"
}

function install_cilium() {
  kubectl -n longhorn-system delete networkpolicy longhorn-networkpolicy --ignore-not-found &>/dev/null || true

  configure_cilium

  local config_path="/etc/rancher/rke2/config.yaml"
  if [[ "$(grep cilium -c ${config_path})" -eq 0 ]]; then
    info "Patching RKE2 configuration"
    sed -i '1s/^/cni: cilium\n/' ${config_path}

    info "Restarting RKE2"
    systemctl restart rke2-server.service
  fi

  info "Waiting for Cilium to be installed"
  local try=0
  local maxtry=120
  
  while ! [ "$(kubectl -n kube-system get ds cilium --ignore-not-found -o name | wc -l)" -gt 0 ] ; do
    if [ $try == $maxtry ]; then
      error "Timed out waiting for Cilium to be installed"
    fi
  
    # When there is no CNI (Canal uninstalled, Cilium not installed), this Webhook can't be reached
    # by kube-api and prevents Cilium installer from running. It is recreated automatically by Rancher.
    kubectl delete mutatingwebhookconfiguration rancher.cattle.io --ignore-not-found
  
    info "Waiting for DaemonSet kube-system/cilium to be created (tries: ${try}/${maxtry})"
    sleep 5
    try=$((try+1))
  done
  info "Cilium is installed"

  info "Waiting for Cilium to be ready"
  try=0
  while true; do
    if [ $try == $maxtry ]; then
      error "Timed out waiting for Cilium to be ready"
    fi

    local desired=0
    local ready=0
    desired=$(kubectl -n kube-system get ds cilium -o jsonpath="{.status.desiredNumberScheduled}")
    ready=$(kubectl -n kube-system get ds cilium -o jsonpath="{.status.numberReady}")
    info "Waiting for DaemonSet kube-system/cilium to be ready (replicas: ${ready}/${desired}, tries: ${try}/${maxtry})"
    [[ "${desired}" -gt "${ready}" ]] || break
    sleep 5
    try=$((try+1))
  done
  info "Cilium is ready"
}