#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/offline-utils.sh"

function is_package_installed() {
  local package="$1"
  local minimum_package_version="$2"
  local installed_package_version
  local semver_compare
  local return_code=0

  if ! installed_package_version=$(rpm -q --queryformat '%{VERSION}' "${package}") ; then
    return_code=1
  else
    if [[ -n $minimum_package_version ]]; then
      semver_compare=$("${MODULES_DIR}/semver-custom.sh" "${installed_package_version}" "${minimum_package_version}")
      [[ $semver_compare == *"error"* ]] || [[ $semver_compare -lt 0 ]] && return_code=1
    fi
  fi
  return "$return_code"
}

#####
# Create the online UiPath Automation Suite repo file
# $1 - package to install
#####
function add_repo() {
  rm -f /etc/yum.repos.d/uipath-as*.repo

  cat >/etc/yum.repos.d/uipath-as.repo <<EOF
[uipath-as]
name=UiPath Automation Suite
baseurl=https://download.uipath.com/automation-suite/rpm/
enabled=0
gpgcheck=0
EOF
}

function install_online_python_package() {
  local pkg="$1"
  if ! python3 -m pip show "$pkg" &>/dev/null; then
    info "installing $pkg."
    python3 -m pip install "$pkg" 2>/dev/null || error_with_code "14015" "$pkg"
  fi
}

function enable_iscsid() {
  #To handle this scenario of iscsid service does not exists
  if [[ "$(systemctl is-enabled iscsid 2>/dev/null)" == "" ]]; then
    error "package iscsi-initiator-utils is required but not installed."
  fi

  if ! systemctl --quiet is-active iscsid; then
    echo "starting iscsid service"
    systemctl enable iscsid && systemctl start iscsid
  fi
}

#####
# Install the prerequisites required for both install and runtime
# ensure iscsid is enabled and stared
#####
function install_package_dependencies() {
  local required_os_pkgs=(jq iscsi-initiator-utils nfs-utils rpcbind nc util-linux openssl httpd-tools crun)
  add_repo
  install_packages "uipath-as" "${required_os_pkgs[@]}"

  enable_iscsid
}


#####
# Install the prerequisites required for running the readiness validation
#####
function install_online_readiness_validation_prereqs() {
  local pkgs_list=(podman bind-utils crun openssl conmon)
  add_repo
  install_packages "uipath-as" "${pkgs_list[@]}"
}

function install_online_ctl_prereqs() {
  local pkgs_list=(sshpass python3 zip unzip)
  add_repo
  install_packages "uipath-as" "${pkgs_list[@]}"

  install_online_python_package "ansible"
}

# This dependency required to run online/offline readiness validation
function install_offline_readiness_validation_prereqs() {
    local readiness_validation_dependencies_folder=$1
    local readiness_validation_prereqs_bundle="${readiness_validation_dependencies_folder}/prereqs.tar.gz"
    local readiness_validation_prereqs_sha="${readiness_validation_dependencies_folder}/prereqs-sha256sum.txt"
    local readiness_validation_prereqs_dir="${readiness_validation_dependencies_folder}/readiness_validation_prereqs"
    local pkgs_list=(podman bind-utils)

    info "Installing readiness validation prerequisites from ${readiness_validation_prereqs_dir}"
    check_bundle "${readiness_validation_prereqs_bundle}" "${readiness_validation_prereqs_sha}"
    mkdir -p "${readiness_validation_prereqs_dir}" && tar -xzf "${readiness_validation_prereqs_bundle}" -C "${readiness_validation_prereqs_dir}"

    cat > /etc/yum.repos.d/uipath_rediness_validation_prereqs.repo <<EOF
[uipath_rediness_validation_prereqs]
name=uipath_rediness_validation_prereqs
baseurl=file://${readiness_validation_prereqs_dir}
enabled=0
gpgcheck=0
EOF
  install_packages "uipath_rediness_validation_prereqs" "${pkgs_list[@]}"

  info "Cleaning up ${readiness_validation_prereqs_dir} & ${readiness_validation_prereqs_bundle}"
  rm -rf "${readiness_validation_prereqs_dir}" "${readiness_validation_prereqs_bundle}"
}
