#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.


lenghty_info_message="This operation may be lengthy. Please allow for it to complete..."
export HELM_EXPERIMENTAL_OCI=1

#####
# Populate docker registry with charts from the offline bundle
# $1 - folder where the bundle was uncompressed
#####
function import_sf_charts() {
  local sf_folder="${1}"
  local chart_inventory="${sf_folder}/helm/helm-tags.txt"

  info "Populating Helm Registry at ${HELM_REGISTRY_URL}"

  [[ -f "${chart_inventory}" ]] || error_with_code "10006" "${chart_inventory}"
  while read -r line; do
    local chart_name file_name
    chart_name=$(awk '{print $1}' <<<"${line}")
    file_name=$(awk '{print $2}' <<<"${line}")
    helm chart save "${sf_folder}/helm/${file_name}" "${HELM_REGISTRY_URL}/${chart_name}"
    local try=0
    local maxtry=10
    local status="notready"
    while [[ ${status} == "notready" ]] && ((try != maxtry)); do
      try=$((try + 1))
      info "Pushing local chart ${sf_folder}/helm/${file_name} to ${chart_name}...try ${try}/${maxtry}" && sleep 2
      helm chart push "${HELM_REGISTRY_URL}/${chart_name}" && status="ready"
    done
    [[ ${status} == "ready" ]] || error_with_code "10019" "${sf_folder}/helm/${file_name}"
  done <"${chart_inventory}"

  info "Populated Helm Registry"
}

#####
# $1 - podman folder
# $2 - offline installation folder
#####
function import_offline_sf_artifacts() {
  local podman_folder="${1}"
  local offline_install_folder="${2}"
  local sf_bundle="${offline_install_folder}/sf-bundle.tar.gz"
  local sf_sha="${offline_install_folder}/sf-bundle-sha256sum.txt"
  local offline_sf_dir="${offline_install_folder}/sf"

  configure_docker_tools "${podman_folder}"

  check_bundle "${sf_bundle}" "${sf_sha}"
  info "Extracting ${sf_bundle} to ${offline_sf_dir}"
  info "${lenghty_info_message}"
  mkdir -p "${offline_sf_dir}" && tar -xzf "${sf_bundle}" -C "${offline_sf_dir}"
  rm -f "${sf_bundle}"

  import_sf_images "${offline_sf_dir}" "${podman_folder}"
  import_sf_charts "${offline_sf_dir}"
}

#####
# perform login to local docker registry for podman and helm
# no parameters are passed
#####
function login_docker_tools() {
  podman login "localhost:${DOCKER_REGISTRY_NODEPORT}" --username "${HELM_REGISTRY_USERNAME}" --password "${HELM_REGISTRY_PASSWORD}" --tls-verify=false || error_with_code "10026" "${DOCKER_REGISTRY_NODEPORT}"

  local try=0
  local maxtry=10
  local status="notready"
  while [[ ${status} == "notready" ]] && ((try != maxtry)); do
    try=$((try + 1))
    info "Helm login to registry at ${HELM_REGISTRY_URL}...try ${try}/${maxtry}"
    helm registry login "${HELM_REGISTRY_URL}" --username "${HELM_REGISTRY_USERNAME}" --password "${HELM_REGISTRY_PASSWORD}" && status="ready"
    [[ ${status} == "ready" ]] || sleep 30
  done
  [[ ${status} == "ready" ]] || error_with_code "10023" "${HELM_REGISTRY_URL}"
}

#####
# configure the podman installation
# create both rootless and rootfull paths
# update the config file (storage.conf) to point to these paths
# update the config file (storage.conf) to uncomment the overlayfs config
# $1 - podman folder
#####
function configure_podman() {
  local podman_folder="$1"
  local podman_config_path="/etc/containers/storage.conf"
  local podman_rootless_path="${podman_folder}/podman_rootless"
  local podman_rootfull_path="${podman_folder}/podman_rootfull"

  if [[ -n "${podman_folder}" ]]; then
    info "Configuring podman to use ${podman_folder}"
    mkdir -p "${podman_rootless_path}" || error_with_code "10031" "${podman_rootless_path}"
    mkdir -p "${podman_rootfull_path}" || error_with_code "10031" "${podman_rootfull_path}"
    # update the graphroot config line
    sed -i "s|^\(graphroot[[:space:]]=\).*|\1\"${podman_rootfull_path}\"|" "${podman_config_path}"

    # uncomment and update the rootless path config line
    sed -i "s|^#[[:space:]]*\(rootless_storage_path[[:space:]]*=\).*|\1\"${podman_rootless_path}\"|" "${podman_config_path}"
  fi
  # uncomment the mount program
  sed -i "/^#mount_program[[:space:]]*=/s|^#||g" "${podman_config_path}"
}

#####
# configure the docker registry tools
# configure podman and perform login into the local docker registry
# $1 - podman folder
#####
function configure_docker_tools() {
  if [[ -n "${1}" ]]; then
    configure_podman "$1"
  fi
  login_docker_tools
}

######
# extract offline bundle
# $1 - path to offline bundle
# $2 - path where the bundle is uncompressed
######
function extract_offline_bundle() {
  local offline_bundle_location="$1"
  local offline_bundle_install_location="$2"
  mkdir -p "${offline_bundle_install_location}"
  info "Extracting ${offline_bundle_location} to ${offline_bundle_install_location}"
  info "This operation may be lengthy. Please allow for it to complete..."
  tar -xzf "${offline_bundle_location}" -C "${offline_bundle_install_location}"
}

#####
# Verify tar gz bundle against the expected SHA sum
# $1 - path where bundle is located
# $2 - path where sha is located
#
####
function check_bundle() {
  local bundle_path=${1}
  local bundle_sha=${2}

  info "Verifying ${bundle_path}"

  [[ -f "${bundle_path}" ]] || error_with_code "10006" "${bundle_path}"
  [[ -f "${bundle_sha}" ]] || error_with_code "10006" "${bundle_sha}"

  SHA_ACTUAL=$(sha256sum "${bundle_path}" | awk '{print $1}')
  SHA_EXPECTED=$(<"${bundle_sha}")

  if [ "${SHA_ACTUAL}" != "${SHA_EXPECTED}" ]; then
    error_with_code "10016" "${bundle_path}"
  fi

  info "${bundle_path} passed the checks"
}

######
# Import on-demand artifacts
# $1 - podman folder
# $2 - offline installation folder
######
function import_on_demand_artifacts() {
  local podman_folder="${1}"
  local offline_bundle_folder="${2}"
  local offline_bundle_dir
  local bundle
  local filename
  local bundle_root

  configure_docker_tools "${podman_folder}"

  for bundle in "${offline_bundle_folder}"/*.tar.gz; do
    filename=$(basename -- "${bundle}")
    echo "${filename}"
    bundle_root="${filename%%.*}"
    offline_bundle_dir="${offline_bundle_folder}/${bundle_root}"
    check_bundle "${offline_bundle_folder}/${bundle_root}.tar.gz" "${offline_bundle_folder}/${bundle_root}-sha256sum.txt"
    info "Extracting ${bundle} to ${offline_bundle_dir}"
    mkdir -p "${offline_bundle_dir}" && tar -xzf "${bundle}" -C "${offline_bundle_dir}"

    import_sf_images "${offline_bundle_dir}" "${podman_folder}"
  done
}

#####
# Populate docker registry with images from the offline bundle
# $1 - folder where the artifacts are located
# $2 - podman folder
#####
function import_sf_images() {
  local sf_folder="${1}"
  local podman_folder="${2}"
  local image_inventory="${sf_folder}/images/image-tags.txt"
  local image_tar="${sf_folder}/images/images.tar"

  info "Populating Docker Registry at https://localhost:${DOCKER_REGISTRY_NODEPORT}"

  if [[ -z "${podman_folder}" ]]; then
    podman_folder="${sf_folder}"
  fi

  [[ -f "${image_inventory}" ]] || error_with_code "10006" "${image_inventory}"
  local all_images_present=1

  while read -r image_name; do
    podman image exists "localhost:${DOCKER_REGISTRY_NODEPORT}/${image_name}" || all_images_present=0
  done <"${image_inventory}"

  mkdir -p "${podman_folder}/tmp"
  info "Executing podman load with podman_folder=${podman_folder}"
  # inline ENV for https://github.com/containers/podman/pull/5412 and https://github.com/containers/podman/issues/5960
  export TMPDIR="${podman_folder}/tmp"

  if ((all_images_present != 1)); then
    info "Loading image tar ${image_tar}"
    info "${lenghty_info_message}"
    podman load -i "${image_tar}" || error_with_code "10017" "${image_tar}"
  else
    info "All images present. Skipping podman load"
  fi
  while read -r image_name; do
    local try=0
    local maxtry=10
    local status="notready"
    while [[ ${status} == "notready" ]] && ((try != maxtry)); do
      try=$((try + 1))
      info "Pushing image ${image_name}...try ${try}/${maxtry}" && sleep 5
      podman push "localhost:${DOCKER_REGISTRY_NODEPORT}/${image_name}" \
        --tls-verify=false >/dev/null &&
        status="ready"
      [[ ${status} == "ready" ]] || sleep 60
    done
    [[ ${status} == "ready" ]] || error_with_code "10018" "${image_name}"
  done <"${image_inventory}"

  info "Populated Docker Registry"
}

#####
# Install the packages in the list, using exclusively the repository received as argument
# $1 - repository name
# $2 - list of packages to install
#####
function install_packages() {
  local repo_name="$1"
  local pkgs_list=("${@:2}")
  local container_tools_enabled="false"
  dnf module list --disablerepo="*" --enablerepo="${repo_name}" --enabled container-tools && container_tools_enabled="true"
  [[ ${container_tools_enabled} == "true" ]] && dnf -y module disable container-tools --disablerepo="*" --enablerepo="${repo_name}"
  for pkg in "${pkgs_list[@]}"; do
    dnf -y install --disablerepo=* --enablerepo="${repo_name}" "${pkg}" --allowerasing
  done
}

#####
# Create the local yum repository, and populate it with packages from the archive
# $1 - folder to create the local repository
# $2 - name of archive
# $3 - name of sha file
# $4 - repository name
# $5 - packages list
#####
function install_offline_rpms() {
  local offline_install_folder="${1}"
  local bundle_path="${2}"
  local bundle_sha="${3}"
  local repo_name="${4}"
  local pkgs_list=("${@:5}")

  info "Installing offline prerequisites from ${bundle_path}"
  check_bundle "${bundle_path}" "${bundle_sha}"
  mkdir -p "${offline_install_folder}" && tar -xzf "${bundle_path}" -C "${offline_install_folder}"

  cat >/etc/yum.repos.d/"${repo_name}".repo <<EOF
[${repo_name}]
name=${repo_name}
baseurl=file://${offline_prereqs_dir}
enabled=0
gpgcheck=0
EOF
  install_packages "${repo_name}" "${pkgs_list[@]}"
  info "Cleaning up temporary folder ${offline_install_folder}"
  rm -rf "${offline_install_folder}"
}

#####
# Install the prerequisites from the prereqs archive
# $1 - folder where the archive and sha are present
#####
function install_offline_prereqs() {
  local offline_install_folder=$1
  local prereqs_bundle="${offline_install_folder}/prereqs.tar.gz"
  local prereqs_sha="${offline_install_folder}/prereqs-sha256sum.txt"
  local offline_prereqs_dir="${offline_install_folder}/prereqs"
  local pkgs_list=(iscsi-initiator-utils jq nfs-utils rpcbind util-linux nc podman openssl httpd-tools)

  install_offline_rpms "${offline_prereqs_dir}" "${prereqs_bundle}" "${prereqs_sha}" "uipath_prereqs" "${pkgs_list[@]}"

  systemctl enable iscsid
  systemctl start iscsid
}

#####
# Install the prerequisites from the prereqs archive
# $1 - folder where the archive and sha are present
#####
function install_offline_ansible_prereqs() {
  local offline_install_folder=$1
  local prereqs_bundle="${offline_install_folder}/ansibleprereqs.tar.gz"
  local prereqs_sha="${offline_install_folder}/ansibleprereqs-sha256sum.txt"
  local offline_prereqs_dir="${offline_install_folder}/ansibleprereqs"
  local pkgs_list=(python3 sshpass zip unzip)

  install_offline_rpms "${offline_prereqs_dir}" "${prereqs_bundle}" "${prereqs_sha}" "uipath_ansible" "${pkgs_list[@]}"
}

#####
# Install the ansible from an archive
# 2 steps: install python prereqs and then install ansible as a python package
# $1 - ansible archive
# $2 - path where the archives will be uncompressed
#####
function install_offline_ansible() {
  local ansible_bundle_location="${1}"
  local ansible_install_folder="${2}"
  local ansible_packages_folder="${ansible_install_folder}/ansible-packages"
  local ansible_packages_bundle="${ansible_install_folder}/ansible-py.tar.gz"

  extract_offline_bundle "${ansible_bundle_location}" "${ansible_install_folder}"
  install_offline_ansible_prereqs "${ansible_install_folder}"


  mkdir -p "${ansible_packages_folder}"
  tar -xzf "${ansible_packages_bundle}" -C "${ansible_packages_folder}"
  python3 -m pip install ansible --no-index --find-links "${ansible_packages_folder}"
}


#####
# Check the offline bundle crc, validating the observed with the expected value
# $1 - offline bundle path
# $2 - offline bundle type
#####
function check_crc() {
  local offline_bundle_path="${1}"
  local offline_bundle_type="${2}"
  local crc_expected
  local crc_actual
  local crc_file_path
  crc_file_path="${SCRIPT_DIR}/versions/bundles-crc.json"
  info "verifying crc for ${offline_bundle_path} of type ${offline_bundle_type}"

  # shellcheck disable=SC2154
  crc_expected="$(jq -r --arg keyName "${offline_bundle_type}" '.[$keyName]' < "${crc_file_path}")"
  if [[ ${crc_expected} == null ]]; then
    error "could not find type ${offline_bundle_type}"
  fi

  crc_actual=$(gzip -lv "${offline_bundle_path}" | tail -1 | awk '{print $2}')
  if [[ "${crc_expected}" != "${crc_actual}" ]]; then
    return 1
  fi
  return 0
}
