#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

#GLOBALS
PROXY_SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${PROXY_SCRIPT_DIR}" ]]; then
  echo "Could not determine script path"
  exit 1
fi

MESSAGE_FILE="${PROXY_SCRIPT_DIR}/../readiness_message_en.json"
if [[ ! -f "${MESSAGE_FILE}" ]] ; then
  echo "Proxy Readiness message file not present"
  exit 1
fi

MODULES_DIR=$(cd "${PROXY_SCRIPT_DIR}/../../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  echo "Could not determine Modules path"
  exit 1
fi

export MODULES_DIR

if [[ -f "${MODULES_DIR}/utils.sh" ]] ; then
  #shellcheck disable=SC1091,SC1090
  source "${MODULES_DIR}/utils.sh"
else
  echo "Common utils not found at ${MODULES_DIR}/utils.sh"
  exit 1
fi

##exit code
EXIT_CODE=0

function display_usage() {
  echo
  echo "***************************************************************************************"
  echo
  echo "Run proxy checks from nodes"
  echo
  echo "Arguments"
  echo "  -d|--debug                         Increase logging verbosity to show all debug logs"
  echo "  -h|--help                          Show this help message and exit"
  echo  
  echo "***************************************************************************************"
  echo
}

function parse_long_args() {
  while (("$#")); do
    case "$1" in
    -d | --debug)
      echo "Running in debug"
      shift
      set -x
      ;;

    -h | --help)
      display_usage
      shift
      exit 0
      ;;

    -*|*) # unsupported flags
      shift
      ;;
    esac
  done
}

function is_empty() {
  local value=$1
  local result=0
  [[ -z "${value}" || "${value}" == " " ]] &&  result=1
  echo ${result}
}

function proxy_server_connectivity_checks() {
  local url="$1"
  local output
  output=$(curl -m 10  -w "%{http_code}" --silent --output /dev/null "${url}")
  if [[ $output == "000" ]] ; then
    add_test_result "FAILED" "$(get_readiness_message ".proxy.accessibility_error" "${MESSAGE_FILE}" )" "$(get_readiness_mitigation ".proxy.accessibility_error" "${MESSAGE_FILE}")"
  else
    add_test_result "PASSED" "$(get_readiness_message ".proxy.accessibility_info" "${MESSAGE_FILE}" )" "$(get_readiness_mitigation ".proxy.accessibility_info" "${MESSAGE_FILE}")"
  fi
}

function compare() {
  local curl_value="$1"
  local wget_value="$2"
  local user_input="$3"
  local result=0
  if [[ $(is_empty "${curl_value}") == "1" ]] || [[ $(is_empty "${wget_value}") == "1" ]] \
      || [[ "${wget_value}" != "${curl_value}" ]] || [[ $(is_empty "${user_input}") == "0" && "${user_input}" != "${curl_value}" ]]; then
    result=1
  fi
  echo ${result}
}

function validate_noproxy_value() {
  local fqdn="$1"
  local curl_value="$2"
  local urls
  declare -a fqdnMatchArray=(
    ["alm"]=0
    ["9345"]=0
    ["6443"]=0
  )
  fqdn="$1"
  curl_value="$2"

  if [[ $(is_empty "${curl_value}") == "0" ]] ; then
    # shellcheck disable=SC2162
    IFS=',' read -a urls <<< "$curl_value"
    for url in "${urls[@]}"
    do
      case "$(echo "${url}" | xargs)" in
        "alm.${fqdn}" )
          fqdnMatchArray["alm"]=1
          ;;
        "${fqdn}:9345" )
          fqdnMatchArray["9345"]=1
        ;;
        "${fqdn}:6443" )
          fqdnMatchArray["6443"]=1
        ;;
        "${fqdn}" )
          fqdnMatchArray["alm"]=1
          fqdnMatchArray["6443"]=1
          fqdnMatchArray["9345"]=1
        ;;
        *)
        ;;
      esac
    done
  fi
  if [[ ${fqdnMatchArray["alm"]} == 0 || ${fqdnMatchArray["6443"]} == 0 || ${fqdnMatchArray["9345"]} == 0 ]]; then
    add_test_result "FAILED" "$(get_readiness_message ".proxy.fqdn_configuration_error" "${MESSAGE_FILE}" "${var_name}" )" "$(get_readiness_mitigation ".proxy.fqdn_configuration_error" "${MESSAGE_FILE}")"
  fi
}

function run_validation() {
  local curl_value wget_value user_input
  local proxy_var_arr=("http_proxy" "https_proxy" "no_proxy")
  
  for var_name in "${proxy_var_arr[@]}"
  do
    wget_value=$(grep "^${var_name}" "/etc/wgetrc" | awk -F= '{print $2}' | xargs)
    curl_value=$(env | grep "^${var_name}" | awk -F= '{print $2}' | xargs )
    user_input="$(get_optional_json_value ".proxy.${var_name}" )"
    if [[ "${var_name}" == "no_proxy" ]]; then
      user_input=""
    fi
    if [[ $(compare "${curl_value}" "${wget_value}" "${user_input}") == "1" ]]; then
      add_test_result "FAILED" "$(get_readiness_message ".proxy.config_error" "${MESSAGE_FILE}" "${var_name}" )" "$(get_readiness_mitigation ".proxy.config_error" "${MESSAGE_FILE}" "${var_name}")"
    else
      add_test_result "PASSED" "$(get_readiness_message ".proxy.config_info" "${MESSAGE_FILE}" "${var_name}" )" "$(get_readiness_mitigation ".proxy.config_info" "${MESSAGE_FILE}")"
    fi
  done

  if [[ ${EXIT_CODE} == 0 ]]; then
    curl_value=$(env | grep "no_proxy" | awk -F= '{print $2}' | xargs )
    validate_noproxy_value "$(get_optional_json_value ".fqdn" )" "${curl_value}"
    curl_value=$(env | grep "^http_proxy" | awk -F= '{print $2}' | xargs )
    proxy_server_connectivity_checks "${curl_value}"
  fi
}

function main() {
  parse_long_args "$@"
  is_proxy_enabled=$(get_optional_json_value ".proxy.enabled")
  print_center "Proxy Validation"
  if [[ -n ${is_proxy_enabled} && "${is_proxy_enabled}" == "true" ]]; then
    #shellcheck disable=SC2005
    echo "$(get_readiness_message ".proxy.start_info" "${MESSAGE_FILE}")"
    run_validation
    echo -e "$(get_readiness_message ".proxy.end_info" "${MESSAGE_FILE}")\n"
  else
    add_test_result "INFO" "$(get_readiness_message ".proxy.skip_warn" "${MESSAGE_FILE}" )" "$(get_readiness_mitigation ".proxy.skip_warn" "${MESSAGE_FILE}")"
    echo -e ""
  fi
  print_center ""
}

main "$@"
exit "${EXIT_CODE}"