#!/bin/bash

# Copyright UiPath 2021
#
# =================
# LICENSE AGREEMENT
# -----------------
#   Use of paid UiPath products and services is subject to the licensing agreement
#   executed between you and UiPath. Unless otherwise indicated by UiPath, use of free
#   UiPath products is subject to the associated licensing agreement available here:
#   https://www.uipath.com/legal/trust-and-security/legal-terms (or successor website).
#   You must not use this file separately from the product it is a part of or is associated with.

set -eu

# kill subprocesses whem the caller script terminates
trap "trap - SIGTERM && jobs -p | xargs -r kill" INT TERM EXIT

#GLOBALS:
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error_with_code "10001"
fi
MODULES_DIR=$(cd "${SCRIPT_DIR}/../Modules" && pwd)
if [[ -z "${MODULES_DIR}" ]]; then
  error_with_code "10002"
fi

readonly SCRIPT_DIR
readonly MODULES_DIR
export MISSING_DEGRADED_APP_LIST=""

## Locale
#shellcheck disable=SC2034
LANG_SERVICES_PREFIX="[Products]"
LANG_SERVICES_STARTED_PREFIX="[Products] [Started]"
LANG_SERVICES_FAILED_PREFIX="[Products] [Failed]"
LANG_SERVICES_COMPLETED_PREFIX="[Products] [Completed]"
LANG_SERVICES_CONFIG_MESSAGE="Generating the config resources"

#shellcheck disable=SC2034
declare -A APPS_STATUS=()
#shellcheck disable=SC2034
declare -i APPS_PROCESSED=0
#shellcheck disable=SC2034
declare -a SERVICE_NAMES_ENABLED
#shellcheck disable=SC2034
declare -a ALL_SERVICE_NAMES
declare -a ADDITIONAL_PARAMETERS
declare -a MANDATORY_PARAMETERS
#shellcheck disable=SC1090,SC1091
source "${MODULES_DIR}/utils.sh"
#shellcheck disable=SC1090,SC1091,SC2034
source "${MODULES_DIR}/argocd_utils.sh"
#shellcheck disable=SC1090,SC1091,SC2034
source "${MODULES_DIR}/cluster_utils.sh"
#shellcheck disable=SC1090,SC1091,SC2034
source "${MODULES_DIR}/k8sutils.sh"
#shellcheck disable=SC1090,SC1091,SC2034
source "${MODULES_DIR}/services_installer_utils.sh"
#FUNCTIONS

function parse_standalone_aicenter_input_file() {
  ORCHESTRATOR_URL="$(get_mandatory_json_value ".aicenter.orchestrator_url")"
  [[ "${ORCHESTRATOR_URL: -1}" != "/" ]] && ORCHESTRATOR_URL+="/"

  IDENTITY_SERVER_URL="$(get_mandatory_json_value ".aicenter.identity_server_url")"
  ORCHESTRATOR_CERT_FILE_PATH="$(get_mandatory_json_value ".aicenter.orchestrator_cert_file_path")"
  ORCHESTRATOR_CERT_FILE_PATH="${ORCHESTRATOR_CERT_FILE_PATH/#\~/$HOME}"
  IDENTITY_CERT_FILE_PATH="$(get_mandatory_json_value ".aicenter.identity_cert_file_path")"
  IDENTITY_CERT_FILE_PATH="${IDENTITY_CERT_FILE_PATH/#\~/$HOME}"
  IDENTITY_ACCESS_TOKEN="$(get_mandatory_json_value ".aicenter.identity_access_token")"
  METERING_API_KEY=""
  if [[ ${INSTALL_TYPE} == 'online' ]];then
    METERING_API_KEY="$(get_mandatory_json_value ".aicenter.metering_api_key")"
    METERING_API_KEY="$(echo -n "$METERING_API_KEY" | base64 -w 0)"
  fi
  ACTION_CENTER_URL="$(get_optional_json_value ".aicenter.action_center_url")"
}

function get_additional_parameters() {
  ADDITIONAL_PARAMETERS=()
    [ -z "${DOCKER_REGISTRY_PULL_SECRET}" ] || ADDITIONAL_PARAMETERS+=("-p global.imagePullSecrets[0]='uipathpullsecret'" )

  ADDITIONAL_PARAMETERS+=(-p global.applications.apps.aicenter.standalone.enabled="true" )
  ADDITIONAL_PARAMETERS+=(-p global.applications.apps.platform.enabled="false" )
  
  #shellcheck disable=SC2034
  SERVICE_NAMES_ENABLED=("aicenter" "documentunderstanding")
  #shellcheck disable=SC2034
  ALL_SERVICE_NAMES=("aicenter" "documentunderstanding" )
}

function read_orch_certificate_files() {
  if [[ -f ${ORCHESTRATOR_CERT_FILE_PATH} ]]; then
    ORCHESTRATOR_CERT_FILE_CONTENT=$(base64 -i "${ORCHESTRATOR_CERT_FILE_PATH}")
    # shellcheck disable=SC2001,2086
    ORCHESTRATOR_CERT_FILE_CONTENT=$(echo ${ORCHESTRATOR_CERT_FILE_CONTENT} | sed -e 's/[[:space:]]//g')
  else
    error "${ORCHESTRATOR_CERT_FILE_PATH} not found"
  fi

  if [[ -f ${IDENTITY_CERT_FILE_PATH} ]]; then
    IDENTITY_CERT_FILE_CONTENT=$(base64 -i "${IDENTITY_CERT_FILE_PATH}")
    # shellcheck disable=SC2001,2086
    IDENTITY_CERT_FILE_CONTENT=$(echo ${IDENTITY_CERT_FILE_CONTENT} | sed -e 's/[[:space:]]//g')
  else
    IDENTITY_CERT_FILE_CONTENT=${ORCHESTRATOR_CERT_FILE_CONTENT}
  fi
}

function install_app_of_apps() {  
  local pid

  get_additional_parameters

  get_mandatory_app_create_parameters
  local try=0
  local maxtry=5
  local status="notready"
  while [[ ${status} == "notready" ]] && (( try != maxtry )) ; do
    try=$((try+1))
    argocd app create uipath \
    --repo "${APP_OF_APPS_REPO}" \
    --revision "${APP_OF_APPS_VERSION}" \
    --helm-chart "helm/${APP_OF_APPS_CHART}" \
    --dest-server https://kubernetes.default.svc \
    --upsert \
    --sync-policy "automated" \
    --sync-retry-backoff-duration 10s \
    --sync-retry-backoff-factor 2 \
    --sync-retry-backoff-max-duration 20m \
    --sync-retry-limit 4 \
    --self-heal \
    "${ADDITIONAL_PARAMETERS[@]}" \
    "${MANDATORY_PARAMETERS[@]}" \
    --values values-aicenter.yaml \
    -p global.userInputs.aicenter.sqlConnectionString="${AICENTER_SQL_CONNECTION:-PLACEHOLDER}" \
    -p global.userInputs.aicenter.orchestratorUrl="${ORCHESTRATOR_URL}" \
    -p global.userInputs.aicenter.identityServerUrl="${IDENTITY_SERVER_URL}" \
    -p global.userInputs.aicenter.orchestratorCert="${ORCHESTRATOR_CERT_FILE_CONTENT}" \
    -p global.userInputs.aicenter.identityCert="${IDENTITY_CERT_FILE_CONTENT}" \
    -p global.userInputs.aicenter.identityAccessToken="${IDENTITY_ACCESS_TOKEN}" \
    -p global.userInputs.aicenter.meteringApiKey="${METERING_API_KEY}" \
    -p global.userInputs.aicenter.actionCenterUrl="${ACTION_CENTER_URL}" \
    -p global.userInputs.documentunderstanding.datamanager.sqlConnectionString="${DU_DATAMANAGER_SQL_CONNECTION:-PLACEHOLDER}" \
    >/dev/null 2>&1 \
    && status="ready"

    [[ ${status} == "ready" ]] || { info "Creating uipath app...try ${try}/${maxtry}"; sleep 5; }
  done
  
  [[ ${status} == "ready" ]] || error_with_code "13001"
  
  info "Application uipath created"

  # sent this to background so we can poll for the apps status  
  sync_with_wait "uipath" "argocd" "app.kubernetes.io/instance=uipath" "service-fabric=service-presync-checks" "uipath" &

  # keep printing the apps status as it changes
  pid="$!"
  while [ -d "/proc/${pid}" ];do
    get_argo_apps_status
    sleep 5
  done
 
  # We need to check and call each unhealthy child application because of argocd limitation
  # https://argo-cd.readthedocs.io/en/stable/user-guide/auto_sync/#automated-sync-semantics
  sync_unhealthy_child_apps "argocd" "uipath"

  # get the last status once the process finished
  get_argo_apps_status
  wait "${pid}" || error_with_code "13002"
}

function set_output() {
  cat > "${OUTFILE}" <<EOF
{ 
  "aicenter_url": "https://${CLUSTER_FQDN}/aifabric_",
  "documentunderstanding_url": "https://${CLUSTER_FQDN}/du_"
}
EOF
}

#MAIN
function main() {
  info "${LANG_SERVICES_STARTED_PREFIX} ${LANG_SERVICES_CONFIG_MESSAGE}"
  # Use trap to append extra error messages to functions imported in this script
  old_trap="$(get_existing_trap "EXIT")"
  #shellcheck disable=SC2064
  trap "error_without_exit \"${LANG_SERVICES_FAILED_PREFIX} ${LANG_SERVICES_CONFIG_MESSAGE}\"; ${old_trap}" EXIT


  cd "${SCRIPT_DIR}"
  parse_args "$@"

  load_os_utils "$SCRIPT_DIR/../Modules"
  
  check_requirements
  
  parse_sf_input_file

  parse_aicenter_input_file

  parse_documentunderstanding_input_file

  parse_standalone_aicenter_input_file


  send_checkpoint_telemetry "AiCenter Services: Input Parsed"

  [[ "$(is_update_fqdn)" == "true" ]] && update_service_start_fqdn_state

  get_fabric_install_paths "${SCRIPT_DIR}"
  get_argocd_install_params
  login_argocd "${ARGOCD_NODEPORT_HTTP}" "${ARGOCD_FQDN}" "${ARGOCD_PASSWORD_SECRET_NAME}" "${ARGOCD_NAMESPACE}" "argocd"

  validate_platform_aicenter_installation_option "aicenter"

  parse_offline_input_parameters

  local try=0
  local maxtry=30
  local status="notready"
  while [[ ${status} == "notready" ]] && (( try != maxtry )) ; do
          try=$((try+1))
          kubectl cluster-info >/dev/null 2>&1  && status="ready"
          info "waiting for cluster to be ready...${try}/${maxtry}" &&  sleep 10;
  done
  kubectl cluster-info || error_with_code "13003"

  declare -a namespaces=("uipath")

  kubectl cluster-info || error_with_code "13003"

  argocd cluster list || error_with_code "13004"

  send_checkpoint_telemetry "AiCenter Services: Cluster Ready"

  create_namespaces "${namespaces[@]}"

  read_orch_certificate_files

  check_available_masters
  
  create_preinstall_override_config

  ensure_cluster_id

  send_checkpoint_telemetry "AiCenter Services: ClusterId Created"
  send_input_telemetry

  #shellcheck disable=SC2064
  trap "${old_trap}" EXIT

  info "${LANG_SERVICES_COMPLETED_PREFIX} ${LANG_SERVICES_CONFIG_MESSAGE}"

  install_app_of_apps

  [[ "$(is_update_fqdn)" == "true" ]] && update_fqdn "${namespaces[@]}"

  set_output

  send_checkpoint_telemetry "AiCenter Services: Success"
  info "UiPath Services installed successfully."
}

main "$@"
