#!/bin/bash
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)

if [[ -z "${SCRIPT_DIR}" ]]; then
    echo "Script dir not present"
    exit 1
fi

#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/utils.sh"
declare -a error_messages

export KUBECONFIG=/etc/rancher/rke2/rke2.yaml PATH=$PATH:/var/lib/rancher/rke2/bin

function validate_deployments_using_keyword() {
  declare -n deployments=$1
  local grep_keyword="$2"
  local error_prefix="$3"
  local namespace="$4"
  declare -n deployments_for_exclusion="$5"
  local -a condition_violating_deploy=()
  for deploy in "${deployments[@]}"; do
    grep_result="$(kubectl get deploy ${deploy} -n ${namespace} -o yaml | grep "${grep_keyword}")"

    if [[ -n "${grep_result}" && "$(array_contains_element deployments_for_exclusion ${deploy})" == "false" ]]; then 
      condition_violating_deploy+=("${deploy}")
    fi
  done

  if [[ ${#condition_violating_deploy[@]} -ne 0 ]]; then 
    error_messages+=("$error_prefix: $(IFS=, ; echo "${condition_violating_deploy[*]}")")
  else
    echo "Validation passed for ${grep_keyword}"
  fi
}

function validate_deployments_not_using_keyword() {
  declare -n deployments=$1
  local grep_keyword="$2"
  local error_prefix="$3"
  local namespace="$4"
  declare -n deployments_for_exclusion="$5"
  local -a condition_violating_deploy=()
  for deploy in "${deployments[@]}"; do
    grep_result="$(kubectl get deploy ${deploy} -n ${namespace} -o yaml | grep "${grep_keyword}")"

    if [[ -z "${grep_result}" && "$(array_contains_element deployments_for_exclusion ${deploy})" == "false" ]]; then 
      condition_violating_deploy+=("${deploy}")
    fi
  done

  if [[ ${#condition_violating_deploy[@]} -ne 0 ]]; then 
    error_messages+=("$error_prefix: $(IFS=, ; echo "${condition_violating_deploy[*]}")")
  fi
}

function validate_deployments_using_auto_reloader() {
  declare -n reloader_deploy_list=$1
  local excluded_deployments
  excluded_deployments=("du-digitizer" "du-frontend" "du-metering" "du-services-ikc" "du-services-hwocr" "du-services-ife" "du-services-fe") 
  validate_deployments_using_keyword reloader_deploy_list "reloader.stakater.com/auto:" "Deployments that are using reloader.stakater.com/auto" "uipath" excluded_deployments
}

function validate_deployments_using_secret_as_env() {
  declare -n secret_deploy_list=$1
  local excluded_deployments
  excluded_deployments=()
  validate_deployments_using_keyword secret_deploy_list "secretKeyRef:" "Deployments that are using secret as environment variables" "uipath" excluded_deployments
}

function validate_deployments_using_configmaps_as_env() {
  declare -n configmaps_env_deploy_list=$1
  local excluded_deployments
  excluded_deployments=("actioncenter-bupproxyservice" "platform-portal" "ai-app-deployment")
  validate_deployments_using_keyword configmaps_env_deploy_list "configMapKeyRef:" "Deployments that are using configmaps as environment variables" "uipath" excluded_deployments
}

function validate_image_pull_secret_for_deployments() {
  declare -n image_pull_secret_deploy_list=$1
  local excluded_deployments
  excluded_deployments=()
  validate_deployments_not_using_keyword image_pull_secret_deploy_list "imagePullSecrets:" "Deployments that are not using imagePullSecret" "uipath" excluded_deployments  
}

function validate_env_section_exists_for_deployments() {
  declare -n env_deployments=$1
  local excluded_deployments
  excluded_deployments=()
  validate_deployments_not_using_keyword env_deployments "env:" "Deployments that are not using env section" "uipath" excluded_deployments  
}

function check_os_in_pods_namespace() {
  local namespace=$1
  PODS=$(kubectl -n "$namespace" get pods -o name)
  VIOLATED_PODS=()
  for pod in $PODS;
  do
    status=$(kubectl -n "$namespace" get "$pod" -o json | jq '.status.phase' | xargs)
    if [[ "${status}" == "Running" ]];
    then
      #shellcheck disable=SC2016
      OS_ID=$(kubectl -n "$namespace" exec -it "$pod" -- bash -c 'source /etc/os-release; echo $ID' | tr -d '\r')
      echo "${pod} ==> ${OS_ID}"
      if [[ "${OS_ID}" != "rhel" ]];
      then
        VIOLATED_PODS+=("${pod}")
      fi
    fi
  done
  echo "Following pods in $namespace namespace do not use rhel based OS ${VIOLATED_PODS[*]}"
}

function validate_rhel_os_pods(){
  check_os_in_pods_namespace "uipath"
  check_os_in_pods_namespace "uipath-infra"
}

function main() {
  mapfile -t uipath_deploy_list < <(kubectl get deploy -n uipath | awk '{print $1}' | tail -n +2)
  validate_deployments_using_auto_reloader uipath_deploy_list
  # This test case does not work for offlineInstaller since we don't specify imagePullSecrets in deployments of offline installer
  # validate_image_pull_secret_for_deployments uipath_deploy_list
  validate_env_section_exists_for_deployments uipath_deploy_list
  validate_deployments_using_configmaps_as_env uipath_deploy_list

  validate_rhel_os_pods

  if [[ ${#error_messages[@]} != 0 ]]; then
    failure_output "Validation Failed" "${error_messages[@]}"
  else
    echo "Validation Passed"
    exit 0
  fi
}